import path from 'path';
import { homedir } from 'os';
import fs from 'fs';
import { checkFileExists, createDir } from '../file/index.ts';
import { ProxyInfo } from '../proxy/proxy.ts';

/**
 * 助手配置文件路径, 全局配置文件目录
 */
const configDir = createDir(path.join(homedir(), '.config/envision/assistant-app'));

/**
 * 助手配置文件初始化
 * @param configRootPath
 * @returns
 */
export const initConfig = (configRootPath: string) => {
  const configDir = createDir(path.join(configRootPath, 'assistant-app'));
  const configPath = path.join(configDir, 'assistant-config.json');
  const appConfigPath = path.join(configDir, 'assistant-app-config.json');
  const appDir = createDir(path.join(configDir, 'frontend'));
  const serviceDir = createDir(path.join(configDir, 'services'));
  const serviceConfigPath = path.join(serviceDir, 'assistant-service-config.json');
  const appPidPath = path.join(configDir, 'assistant-app.pid');
  return {
    /**
     * 助手配置文件路径
     */
    configDir,
    /**
     * 助手配置文件路径, assistant-config.json
     */
    configPath,
    /**
     * 服务目录， 后端服务目录
     */
    serviceDir,
    /**
     * 服务配置文件路径 assistant-service-config.json
     */
    serviceConfigPath,
    /**
     * 应用目录， 前端应用目录
     */
    appDir,
    /**
     * 应用配置文件路径, assistant-app-config.json
     */
    appConfigPath,
    /**
     * 应用进程pid文件路径
     */
    appPidPath,
  };
};
export type ReturnInitConfigType = ReturnType<typeof initConfig>;

type AssistantConfigData = {
  pageApi?: string; // https://kevisual.silkyai.cn
  proxy?: { user: string; key: string; path: string }[];
  apiProxyList?: ProxyInfo[];
  description?: string;
};
let assistantConfig: AssistantConfigData;
type AssistantConfigOptions = {
  configDir?: string;
  init?: boolean;
};
export class AssistantConfig {
  config: AssistantConfigData;
  configPath: ReturnInitConfigType;
  configDir: string;
  constructor(opts?: AssistantConfigOptions) {
    this.configDir = opts?.configDir || configDir;
    if (opts?.init) {
      this.init();
    }
  }
  init() {
    this.configPath = initConfig(this.configDir);
  }
  getConfig() {
    try {
      if (!checkFileExists(this.configPath.configPath)) {
        fs.writeFileSync(this.configPath.configPath, JSON.stringify({ proxy: [] }, null, 2));
        return {
          pageApi: '',
          proxy: [],
        };
      }
      assistantConfig = JSON.parse(fs.readFileSync(this.configPath.configPath, 'utf8'));
      return assistantConfig;
    } catch (error) {
      console.error('file read', error.message);
      return {
        pageApi: '',
        proxy: [],
      };
    }
  }
  getCacheAssistantConfig() {
    if (this.config) {
      return this.config;
    }
    return this.getConfig();
  }
  /**
   * 设置 assistant-config.json 配置
   * @param config
   * @returns
   */
  setConfig(config?: AssistantConfigData) {
    const myConfig = this.getCacheAssistantConfig();
    const newConfig = { ...myConfig, ...config };
    this.config = newConfig;
    fs.writeFileSync(this.configPath.configPath, JSON.stringify(newConfig, null, 2));
    return newConfig;
  }
  /**
   * 应用配置
   * @returns
   */
  getAppConfig(): AppConfig {
    const { appConfigPath } = this.configPath;
    if (!checkFileExists(appConfigPath)) {
      return {
        list: [],
      };
    }
    return JSON.parse(fs.readFileSync(appConfigPath, 'utf8'));
  }
  setAppConfig(config?: AppConfig) {
    const _config = this.getAppConfig();
    const _saveConfig = { ..._config, ...config };
    const { appConfigPath } = this.configPath;

    fs.writeFileSync(appConfigPath, JSON.stringify(_saveConfig, null, 2));
    return _saveConfig;
  }
  assAppConfig(app: any) {
    const config = this.getAppConfig();
    const assistantConfig = this.getCacheAssistantConfig();
    const _apps = config.list;
    const _proxy = assistantConfig.proxy || [];
    const { user, key } = app;
    const newProxyInfo = {
      user,
      key,
      path: `/${user}/${key}`,
    };
    const _proxyIndex = _proxy.findIndex((_proxy: any) => _proxy.path === newProxyInfo.path);
    if (_proxyIndex !== -1) {
      _proxy[_proxyIndex] = newProxyInfo;
    } else {
      _proxy.push(newProxyInfo);
    }

    const _app = _apps.findIndex((_app: any) => _app.id === app.id);
    if (_app !== -1) {
      _apps[_app] = app;
    } else {
      _apps.push(app);
    }
    this.setAppConfig({ ...config, list: _apps });
    this.setConfig({ ...assistantConfig, proxy: _proxy });
    return config;
  }
  getAppList() {
    return this.getAppConfig().list;
  }
}

type AppConfig = {
  list: any[];
};
